// =========================================================
// ECE 4760: Final Project (Fall 2018)
// Professor Bruce Land
// Authors:
//              Stewart Aslan(saa243) 
//          Thinesiya Krishnathasan(tk455) 
//                Yijia Chen(yc2366)
//
// IMU: Desmond Caulley, Nadav Nehoran, Sherry Zhao
// nRF24: Douglas Katz, Fred Kummer
// =========================================================

#include "config_1_2_2a.h"
// threading library
#include "pt_cornell_1_2_2a.h"


#include "config.h"
#include "tft_gfx.h"
#include "tft_master.h"
#include <stdio.h>
#define	SYS_FREQ 40000000 // change the frequency of the clock
#include "nrf24l01.h"
// These two defines are for the tft
#define spi_channel	1
#define spi_divider 10

static char array[32];
static struct pt pt_tx;

static char buffer[120];


static int sel = 0;

static int start_sig;

int adc_11 = 0;

// ====================================================================
// set up the radio in transmit mode
// ====================================================================
void radioSetup_TX() {
    nrf_setup(); // initializing function

    nrf_set_arc(0x05); // 5 retransmits

    nrf_set_ard(0x00); // 250 us between retransmission

    nrf_set_rf_ch(0x01); // freq = 2.401 GHz

    nrf_en_dpl(0); // enable dynamic payloads for transmitting

    // set up addresses for autoack mode and dynamic payload length (dpl) mode
    // The tx address and the pipe 0 rx address must be the same for autoack mode
    // and dpl mode
    nrf_set_address_width(3);
    nrf_set_rx_addr(0, 0xAABBCC, 3);
    nrf_set_tx_addr(0xAABBCC);
}



// ====================================================================
// set up the radio in receive mode
// ====================================================================
void radioSetup_RX() {
    nrf_setup(); // initializing function

    nrf_set_rf_ch(0x01); // freq = 2.401 GHz

    nrf_en_dpl(1); // enable dpl for pipe 1

    // set up addresses for autoack mode and dynamic payload length (dpl) mode
    // The radio will receive on pipe 1
    nrf_set_address_width(3);
    nrf_set_rx_addr(1, 0xAABBCC, 3);
}



static int F_time;

// =============================================================
//  use ADC to read potentiometer 
// =============================================================
void adc_read(){

    tft_setCursor(0, 270);
    tft_setTextColor(ILI9340_WHITE);
    sprintf(buffer, "Set Flight Time:");
    tft_writeString(buffer);
            
    F_time = (ReadADC10(1)/68) + 15;
        
            tft_fillRoundRect(10, 300, 240, 20, 1, ILI9340_BLACK);
            tft_setCursor(10, 300);
            tft_setTextColor(ILI9340_WHITE);
            sprintf(buffer, "%d",F_time);
            tft_writeString(buffer);
}




void main(void) {
    INTEnableSystemMultiVectoredInt();
    
    
    // ======================= setup ADC ==========================================
    // configure and enable the ADC
    CloseADC10(); // ensure the ADC is off before setting the configuration

    // define setup parameters for OpenADC10
    // Turn module on | ouput in integer | trigger mode auto | enable autosample
    // ADC_CLK_AUTO -- Internal counter ends sampling and starts conversion (Auto convert)
    // ADC_AUTO_SAMPLING_ON -- Sampling begins immediately after last conversion completes; SAMP bit is automatically set
    // ADC_AUTO_SAMPLING_OFF -- Sampling begins with AcquireADC10();
#define PARAM1  ADC_FORMAT_INTG16 | ADC_CLK_AUTO | ADC_AUTO_SAMPLING_ON //

    // define setup parameters for OpenADC10
    // ADC ref external  | disable offset test | disable scan mode | do 1 sample | use single buf | alternate mode off
#define PARAM2  ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_ON | ADC_SAMPLES_PER_INT_2 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF
    //
    // Define setup parameters for OpenADC10
    // use peripherial bus clock | set sample time | set ADC clock divider
    // ADC_CONV_CLK_Tcy2 means divide CLK_PB by 2 (max speed)
    // ADC_SAMPLE_TIME_5 seems to work with a source resistance < 1kohm
#define PARAM3 ADC_CONV_CLK_PB | ADC_SAMPLE_TIME_15 | ADC_CONV_CLK_Tcy 

    // define setup parameters for OpenADC10
    // set AN11 and  as analog inputs
//#define PARAM4	ENABLE_AN11_ANA | ENABLE_AN1_ANA // 
#define PARAM4	ENABLE_AN11_ANA // 
    // define setup parameters for OpenADC10
    // DO not skip the channels you want to scan
    // do not specify channels  5 and 11
#define PARAM5	SKIP_SCAN_AN0 | SKIP_SCAN_AN1 | SKIP_SCAN_AN5 | SKIP_SCAN_AN2 | SKIP_SCAN_AN3 | SKIP_SCAN_AN4 | SKIP_SCAN_AN6 | SKIP_SCAN_AN7 | SKIP_SCAN_AN8 | SKIP_SCAN_AN9 | SKIP_SCAN_AN10 | SKIP_SCAN_AN12 | SKIP_SCAN_AN13 | SKIP_SCAN_AN14 | SKIP_SCAN_AN15

    // use ground as neg ref for A 
    // actual channel number is specified by the scan list
    SetChanADC10(ADC_CH0_NEG_SAMPLEA_NVREF); // 
    OpenADC10(PARAM1, PARAM2, PARAM3, PARAM4, PARAM5); // configure ADC using the parameters defined above

    EnableADC10(); // Enable the ADC
    // ====================================================================================
    
    
    // ============== setup TFT =================================================
    tft_init_hw(); // setup for tft
    tft_begin(); // setup for tft
    tft_fillScreen(ILI9340_BLACK);
    //240x320 vertical display
    tft_setRotation(0); // Use tft_setRotation(1) for 320x240
    tft_fillScreen(ILI9340_BLACK); // clear the display
    // ==========================================================================

    char buffer[120]; // a buffer for writing to the tft
    // arrays to read payloads into
    char array[32];
    char sig[32] = {1};
    char array_yaccel[32];
    int toggle = 0;

    int width; // the width of the received payload

    


    // === set up push button analog read on RB3 ===============
    mPORTBSetPinsDigitalIn(BIT_3);

    

 
    radioSetup_TX(); // setup the radio in TX mode
    start_sig = 0;

    
    while (1) {
        
        // Wait for start switch to be toggled
        while (!start_sig) {

            // read the potentiometer
            adc_read();

            tft_setTextColor(ILI9340_RED);
            tft_setTextSize(2);
            tft_setCursor(0, 20);
            sprintf(buffer, "%s", "TX Mode");
            tft_writeString(buffer);
            
            tft_setTextColor(ILI9340_GREEN);
            tft_setTextSize(2);
            tft_setCursor(200,0);
            sprintf(buffer, "%d", mPORTBReadBits(BIT_3));
            tft_writeString(buffer);
     
      
            // if start switch is toggled
            if (mPORTBReadBits(BIT_3)) {
                start_sig = 1;
               
                // transmit the flight time
                sprintf(array, "%d", F_time);
                while (!nrf_send_payload(&array, 32)); // send 32 bytes of data
                nrf_delay_ms(1000);
                
                tft_fillRoundRect(0, 40, 40, 15, 1, ILI9340_BLACK);
                tft_setTextColor(ILI9340_GREEN);
                tft_setTextSize(2);
                tft_setCursor(0, 40);
                sprintf(buffer, "%s", "Transmit Start...");
                tft_writeString(buffer);
                
                radioSetup_RX(); // setup the radio for this program

                tft_fillRoundRect(0, 0, 240, 100, 1, ILI9340_BLACK);
            }


        }
        
        // go into recieve mode
        tft_setTextColor(ILI9340_GREEN);
        tft_setTextSize(2);
        tft_setCursor(0, 0);
        sprintf(buffer, "%s", "RX Mode");
        tft_writeString(buffer);
        // enter receive mode to search for packets
        nrf_state_rx_mode();
        if(nrf_payload_available()){ // wait for a payload to be received
            width = nrf_get_width(); // get the width of the payload
            
            nrf_get_payload(&array, width); // read the payload into the array
            
            
            // display the received data on the tft:
            
            tft_setTextColor(ILI9340_BLUE);
            tft_setTextSize(2);
            tft_setCursor(0, 20);
            sprintf(buffer, "%s", "Received data:");
            tft_writeString(buffer);
            
            tft_setCursor(40, 60);
            tft_setTextColor(ILI9340_RED);
            sprintf(buffer, "%s", "Width:");
            tft_writeString(buffer);
            
            tft_fillRoundRect(110, 60, 140, 14, 1, ILI9340_BLACK);
            tft_setCursor(110, 60);
            sprintf(buffer, "%d", width);
            tft_writeString(buffer);
            tft_setTextColor(ILI9340_BLUE);
              
            tft_setCursor(0, 80);
            tft_setTextColor(ILI9340_RED);
            sprintf(buffer, "%s", "temp:");
            tft_writeString(buffer); 
                        
            tft_setCursor(0, 100);
            tft_setTextColor(ILI9340_RED);
            sprintf(buffer, "%s", "xGyro:");
            tft_writeString(buffer);  
            
            tft_setCursor(0, 120);
            tft_setTextColor(ILI9340_RED);
            sprintf(buffer, "%s", "yGyro:");
            tft_writeString(buffer); 
            
             tft_setCursor(0, 140);
            tft_setTextColor(ILI9340_RED);
            sprintf(buffer, "%s", "zGyro:");
            tft_writeString(buffer); 
            
            tft_setCursor(0, 160);
            tft_setTextColor(ILI9340_RED);
            sprintf(buffer, "%s", "Flight Time:");
            tft_writeString(buffer); 
            

            // resynchronize to make sure that the data is printed in the correct order
            if(width==1) sel = 4;
            
            else{
            
            if(sel == 0){ 
                tft_fillRoundRect(110, 80, 140, 14, 1, ILI9340_BLACK);
                tft_setCursor(110, 80);
                tft_setTextColor(ILI9340_BLUE);
                sprintf(buffer, array);
                tft_writeString(buffer);
              
            }
            
              
             
            if(sel == 1){ 
                tft_fillRoundRect(110, 100, 140, 14, 1, ILI9340_BLACK);
                tft_setCursor(110, 100);
                tft_setTextColor(ILI9340_BLUE);
                sprintf(buffer, array);
                tft_writeString(buffer);
              
            }
            
            
            if(sel == 2){ 
                tft_fillRoundRect(110, 120, 140, 14, 1, ILI9340_BLACK);
                tft_setCursor(110, 120);
                tft_setTextColor(ILI9340_BLUE);
                sprintf(buffer, array);
                tft_writeString(buffer);              
            }
            
            if(sel == 3){ 
                tft_fillRoundRect(110, 140, 140, 14, 1, ILI9340_BLACK);
                tft_setCursor(110, 140);
                tft_setTextColor(ILI9340_BLUE);
                sprintf(buffer, array);
                tft_writeString(buffer);              
            }
            
            if(sel == 4){ 
                tft_fillRoundRect(110, 180, 140, 14, 1, ILI9340_BLACK);
                tft_setCursor(110, 180);
                tft_setTextColor(ILI9340_GREEN);
                sprintf(buffer, array);
                tft_writeString(buffer);              
            }
            }
            
            sel=(sel+1)%5;
        }
        
        
        // if the switch is toggled back down reset the screen
        if ( start_sig == 1 && !mPORTBReadBits(BIT_3)){
            start_sig =0;
            tft_fillRoundRect(0, 0, 240, 320, 1, ILI9340_BLACK);
            radioSetup_TX(); // setup the radio back to TX mode
        }
        
    }
}