#include "config.h"
#include <stdio.h>
#define	SYS_FREQ 40000000 // change the frequency of the clock
#include "nrf24l01.h"
#define spi_divider 10

#include "config_1_2_2a.h"
#include "pt_cornell_1_2_2a.h"
#include "i2c_helper.h"

static struct pt pt_TX;

static int key_start = 1;


//The actual period of the wave
int generate_period = 50000;
static int pwm_low_time = 2500;
int pwm_on_time = 2000;

float values[7] = {0, 0, 0, 0, 0, 0, 0};
static float yAccel = 0;
static float zAccel = 0;
static float xAccel = 0;
static float temp = 0;
static float xGyro = 0;
static float yGyro = 0;
static float zGyro = 0;
static float x_offset = 0;
static float y_offset = 0;
static float xGyro_1; // offset
static float yGyro_1;
static float xGyro_2;
static float yGyro_2;
static int time_now = 0;
static int motor_switch = 1;

static int prop_control_x = 0;
static int d_control_x = 0;
static int control_x = 0;
static int prop_control_y = 0;
static int Kp = 1;
static int Kd = 0;

static int pwm_xplus = 0;
static int pwm_xminus = 0;
static int pwm_yplus = 0;
static int pwm_yminus = 0;

static int wx_list[5] = {0, 0, 0, 0, 0};


void __ISR(_TIMER_3_VECTOR, ipl2) Timer3Handler(void) {
   // clear the timer interrupt flag
    mT3ClearIntFlag();
    
    readImuValues(values);   
    
    // Parse the IMU data
    yAccel = values[1];
    zAccel = values[2];
    temp = values[3];
    xGyro = values[4] - x_offset; // calibrate using offsets
    yGyro = values[5] - y_offset; // calibrate using offsets
    zGyro = values[6];
    
    wx_list[0] = wx_list[1];
    wx_list[1] = wx_list[2];
    wx_list[2] = wx_list[3];
    wx_list[3] = wx_list[4];
    wx_list[4] = xGyro;

    prop_control_x = Kp*wx_list[4]; // drive xGyro to zero
    d_control_x = Kd*(wx_list[4]-wx_list[0]);
    control_x = prop_control_x + d_control_x;
    prop_control_y = Kp*yGyro; // drive yGyro to zero

    if (control_x > 1250) {
        control_x = 1250;
    } else if (control_x < -1250) {
        control_x = -1250;
    }

    if (prop_control_y > 1250) {
        prop_control_y = 1250;
    } else if (prop_control_y < -1250) {
        prop_control_y = -1250;
    }

    pwm_xplus = 2500 + (1250 + control_x) * motor_switch;
    pwm_xminus = 2500 + (1250 - control_x) * motor_switch;
    pwm_yplus = 2500 + (1250 + prop_control_y) * motor_switch;
    pwm_yminus = 2500 + (1250 - prop_control_y) * motor_switch;

    //    if (motor_switch == 1){
            SetDCOC1PWM(pwm_xplus);
            SetDCOC2PWM(pwm_xminus);
            SetDCOC4PWM(2500);
            SetDCOC5PWM(2500);
    //    }
    //    
    //    else if (motor_switch == 0){
    //        SetDCOC1PWM(2500);
    //        SetDCOC2PWM(2500);
    //        SetDCOC4PWM(2500);
    //        SetDCOC5PWM(2500);
    //    }

//    SetDCOC1PWM(pwm_xplus);
//    SetDCOC2PWM(pwm_xminus);
//    SetDCOC4PWM(2500);
//    SetDCOC5PWM(2500);
    //    SetDCOC1PWM(3750);
    //    SetDCOC2PWM(3750);

    //    SetDCOC4PWM(pwm_yplus);
    //    SetDCOC5PWM(pwm_yminus);

    //    SetDCOC1PWM(3750*motor_switch);
    //    SetDCOC2PWM(3750*motor_switch);
    //    SetDCOC4PWM(3750*motor_switch);
    //    SetDCOC5PWM(3750*motor_switch);
}

/* 
 * This code sends data of different sizes using the dynamic payload length 
 * (dpl) feature of the radio. This file is the transmitter and will only send 
 * data. The other file is the receiver and will receive the data of each array 
 * and display it. This code is made to demonstrate sending multiple bytes at 
 * once and sending dynamic length payloads.
 */

// set up the radio

void radioSetup() {
    nrf_setup(); // initializing function

    nrf_set_arc(0x05); // 5 retransmits

    nrf_set_ard(0x00); // 250 us between retransmission

    nrf_set_rf_ch(0x01); // freq = 2.401 GHz

    nrf_en_dpl(0); // enable dynamic payloads for transmitting

    // set up addresses for autoack mode and dynamic payload length (dpl) mode
    // The tx address and the pipe 0 rx address must be the same for autoack mode
    // and dpl mode
    nrf_set_address_width(3);
    nrf_set_rx_addr(0, 0xAABBCC, 3);
    nrf_set_tx_addr(0xAABBCC);
}

// Fill an array with with consecutive numbers
// This is so there is something to look at on the receiving end.

void fillArray(char * array, int len) {
    int i;
    for (i = 0; i < len; i++) {
        array[i] = i;
    }
}

static PT_THREAD(protothread_TX(struct pt *pt)) {
    PT_BEGIN(pt);
    while (1) {
        char array[32];
        
       
        
        sprintf(array, "%d", key_start);
        while (!nrf_send_payload(&array, 1)); // send 32 bytes of data
        nrf_delay_ms(1000);
        
        sprintf(array, "%f", (temp/340)+36.53 );
        while (!nrf_send_payload(&array, 32)); // send 32 bytes of data
        nrf_delay_ms(1000);

        sprintf(array, "%f", xGyro );
        while (!nrf_send_payload(&array, 32)); // send 32 bytes of data
        nrf_delay_ms(1000);
        
        sprintf(array, "%f", yGyro);
        while (!nrf_send_payload(&array, 32)); // send 32 bytes of data
        nrf_delay_ms(1000);
        
        sprintf(array, "%f", zGyro);
        while (!nrf_send_payload(&array, 32)); // send 32 bytes of data
        nrf_delay_ms(1000);

//        char array[32];
//        // Fill the array with consecutive incrementing numbers
//        fillArray(&array,32);
//        while (!nrf_send_payload(&array, 1)); // send one byte of data
//        nrf_delay_ms(1000); // wait a second between each transmission
//        while (!nrf_send_payload(&array, 5)); // send five bytes of data
//        nrf_delay_ms(1000);
//        while (!nrf_send_payload(&array, 10)); // send ten bytes of data
//        nrf_delay_ms(1000);
//        while (!nrf_send_payload(&array, 32)); // send 32 bytes of data
//        nrf_delay_ms(1000);
        
        
    }
    PT_END(pt);
}

void main(void) {

    // =============set up i2c================================
    OpenI2C1(I2C_ON, 40);
    char data[] = {0};
    i2c_write(0x6b, data, 1);
    i2c_write(0x1b, data, 1);

    delay_ms(100);

    readImuValues(values);

    // Parse the IMU data
    yAccel = values[1];
    zAccel = values[2];
    temp = values[3];
    xGyro = values[4];
    yGyro = values[5];
    zGyro = values[6];

    xGyro_1 = xGyro;
    yGyro_1 = yGyro;

    delay_ms(50);

    readImuValues(values);

    // Parse the IMU data
    yAccel = values[1];
    zAccel = values[2];
    temp = values[3];
    xGyro = values[4];
    yGyro = values[5];
    zGyro = values[6];

    xGyro_2 = xGyro;
    yGyro_2 = yGyro;

    // zero the gyro values
    x_offset = (xGyro_2 + xGyro_1) / 2;
    y_offset = (yGyro_2 + yGyro_1) / 2;
    

    // === config the uart, DMA, vref, timer5 ISR ===========
    PT_setup();


    
    

    radioSetup(); // setup the radio for this program
    //240x320 vertical display
    tft_setRotation(0); // Use tft_setRotation(1) for 320x240
    /* array of data that will be sent.
     * the maximum data length that can be sent in one packet is 32 bytes. */
    
    // Fill the array with consecutive incrementing numbers
    //    fillArray(&array,32);
    

    // === Config timer and output compares to make pulses ========
    // set up timer2 to generate the wave period
    OpenTimer3(T3_ON | T3_SOURCE_INT | T3_PS_1_16, generate_period);
    ConfigIntTimer3(T3_INT_ON | T3_INT_PRIOR_2);
    mT3ClearIntFlag(); // and clear the interrupt flag
    
    // set up compare1 for PWM mode, X+ motor
    OpenOC1(OC_ON | OC_TIMER3_SRC | OC_PWM_FAULT_PIN_DISABLE, 0, 0); //
    // OC1 is PPS group 1, map to RPB7 (pin 16)
    PPSOutput(1, RPB7, OC1);

    // set up compare2 for PWM mode, X- motor
    OpenOC2(OC_ON | OC_TIMER3_SRC | OC_PWM_FAULT_PIN_DISABLE, 0, 0); //
    // OC2 is PPS group 2, map to RPB5 (pin 14)
    PPSOutput(2, RPB1, OC2); //B5
    //
    //    // set up compare3 for PWM mode, Y+ motor
    OpenOC4(OC_ON | OC_TIMER3_SRC | OC_PWM_FAULT_PIN_DISABLE, 0, 0); //
    // OC3 is PPS group 4, map to RPB9 (pin 18)
    PPSOutput(3, RPA2, OC4); //RA2
    ////    
    //    // set up compare3 for PWM mode, Y- motor
    OpenOC5(OC_ON | OC_TIMER3_SRC | OC_PWM_FAULT_PIN_DISABLE, 0, 0); //
    // OC3 is PPS group 4, map to RPB9 (pin 18)
    PPSOutput(3, RPB13, OC5); //RA4

    SetDCOC1PWM(2500);
    SetDCOC2PWM(2500);
    SetDCOC4PWM(2500);
    SetDCOC5PWM(2500);
    
    delay_ms(5000);
    
    // === setup system wide interrupts  ====================
    INTEnableSystemMultiVectoredInt();
    
    
    // init the threads
    PT_INIT(&pt_TX);

//    char array[32];
//    sprintf(array, "%d", key_start);
//    while (!nrf_send_payload(&array, 1)); // send 32 bytes of data
//    nrf_delay_ms(1000);
    
    while (1) { // send varying amounts of data

        PT_SCHEDULE(protothread_TX(&pt_TX));

    }
} // main

// === end  ======================================================